/*******************************************************************************
* File Name: TimerOverflow_PM.c
* Version 2.80
*
*  Description:
*     This file provides the power management source code to API for the
*     Timer.
*
*   Note:
*     None
*
*******************************************************************************
* Copyright 2008-2017, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include "TimerOverflow.h"

static TimerOverflow_backupStruct TimerOverflow_backup;


/*******************************************************************************
* Function Name: TimerOverflow_SaveConfig
********************************************************************************
*
* Summary:
*     Save the current user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TimerOverflow_backup:  Variables of this global structure are modified to
*  store the values of non retention configuration registers when Sleep() API is
*  called.
*
*******************************************************************************/
void TimerOverflow_SaveConfig(void) 
{
    #if (!TimerOverflow_UsingFixedFunction)
        TimerOverflow_backup.TimerUdb = TimerOverflow_ReadCounter();
        TimerOverflow_backup.InterruptMaskValue = TimerOverflow_STATUS_MASK;
        #if (TimerOverflow_UsingHWCaptureCounter)
            TimerOverflow_backup.TimerCaptureCounter = TimerOverflow_ReadCaptureCount();
        #endif /* Back Up capture counter register  */

        #if(!TimerOverflow_UDB_CONTROL_REG_REMOVED)
            TimerOverflow_backup.TimerControlRegister = TimerOverflow_ReadControlRegister();
        #endif /* Backup the enable state of the Timer component */
    #endif /* Backup non retention registers in UDB implementation. All fixed function registers are retention */
}


/*******************************************************************************
* Function Name: TimerOverflow_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TimerOverflow_backup:  Variables of this global structure are used to
*  restore the values of non retention registers on wakeup from sleep mode.
*
*******************************************************************************/
void TimerOverflow_RestoreConfig(void) 
{   
    #if (!TimerOverflow_UsingFixedFunction)

        TimerOverflow_WriteCounter(TimerOverflow_backup.TimerUdb);
        TimerOverflow_STATUS_MASK =TimerOverflow_backup.InterruptMaskValue;
        #if (TimerOverflow_UsingHWCaptureCounter)
            TimerOverflow_SetCaptureCount(TimerOverflow_backup.TimerCaptureCounter);
        #endif /* Restore Capture counter register*/

        #if(!TimerOverflow_UDB_CONTROL_REG_REMOVED)
            TimerOverflow_WriteControlRegister(TimerOverflow_backup.TimerControlRegister);
        #endif /* Restore the enable state of the Timer component */
    #endif /* Restore non retention registers in the UDB implementation only */
}


/*******************************************************************************
* Function Name: TimerOverflow_Sleep
********************************************************************************
*
* Summary:
*     Stop and Save the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TimerOverflow_backup.TimerEnableState:  Is modified depending on the
*  enable state of the block before entering sleep mode.
*
*******************************************************************************/
void TimerOverflow_Sleep(void) 
{
    #if(!TimerOverflow_UDB_CONTROL_REG_REMOVED)
        /* Save Counter's enable state */
        if(TimerOverflow_CTRL_ENABLE == (TimerOverflow_CONTROL & TimerOverflow_CTRL_ENABLE))
        {
            /* Timer is enabled */
            TimerOverflow_backup.TimerEnableState = 1u;
        }
        else
        {
            /* Timer is disabled */
            TimerOverflow_backup.TimerEnableState = 0u;
        }
    #endif /* Back up enable state from the Timer control register */
    TimerOverflow_Stop();
    TimerOverflow_SaveConfig();
}


/*******************************************************************************
* Function Name: TimerOverflow_Wakeup
********************************************************************************
*
* Summary:
*  Restores and enables the user configuration
*
* Parameters:
*  void
*
* Return:
*  void
*
* Global variables:
*  TimerOverflow_backup.enableState:  Is used to restore the enable state of
*  block on wakeup from sleep mode.
*
*******************************************************************************/
void TimerOverflow_Wakeup(void) 
{
    TimerOverflow_RestoreConfig();
    #if(!TimerOverflow_UDB_CONTROL_REG_REMOVED)
        if(TimerOverflow_backup.TimerEnableState == 1u)
        {     /* Enable Timer's operation */
                TimerOverflow_Enable();
        } /* Do nothing if Timer was disabled before */
    #endif /* Remove this code section if Control register is removed */
}


/* [] END OF FILE */
