/*******************************************************************************
* File Name: RefCounter.c  
* Version 3.0
*
*  Description:
*     The Counter component consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "RefCounter.h"

uint8 RefCounter_initVar = 0u;


/*******************************************************************************
* Function Name: RefCounter_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void RefCounter_Init(void) 
{
        #if (!RefCounter_UsingFixedFunction && !RefCounter_ControlRegRemoved)
            uint8 ctrl;
        #endif /* (!RefCounter_UsingFixedFunction && !RefCounter_ControlRegRemoved) */
        
        #if(!RefCounter_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 RefCounter_interruptState;
        #endif /* (!RefCounter_UsingFixedFunction) */
        
        #if (RefCounter_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            RefCounter_CONTROL &= RefCounter_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (CY_PSOC5A)
                RefCounter_CONTROL2 &= ((uint8)(~RefCounter_CTRL_MODE_MASK));
            #endif /* (CY_PSOC5A) */
            #if (CY_PSOC3 || CY_PSOC5LP)
                RefCounter_CONTROL3 &= ((uint8)(~RefCounter_CTRL_MODE_MASK));                
            #endif /* (CY_PSOC3 || CY_PSOC5LP) */
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (RefCounter_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                RefCounter_CONTROL |= RefCounter_ONESHOT;
            #endif /* (RefCounter_RunModeUsed != 0x0u) */
            
            /* Set the IRQ to use the status register interrupts */
            RefCounter_CONTROL2 |= RefCounter_CTRL2_IRQ_SEL;
            
            /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
            RefCounter_RT1 &= ((uint8)(~RefCounter_RT1_MASK));
            RefCounter_RT1 |= RefCounter_SYNC;     
                    
            /*Enable DSI Sync all all inputs of the Timer*/
            RefCounter_RT1 &= ((uint8)(~RefCounter_SYNCDSI_MASK));
            RefCounter_RT1 |= RefCounter_SYNCDSI_EN;

        #else
            #if(!RefCounter_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = RefCounter_CONTROL & ((uint8)(~RefCounter_CTRL_CMPMODE_MASK));
            RefCounter_CONTROL = ctrl | RefCounter_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = RefCounter_CONTROL & ((uint8)(~RefCounter_CTRL_CAPMODE_MASK));
            
            #if( 0 != RefCounter_CAPTURE_MODE_CONF)
                RefCounter_CONTROL = ctrl | RefCounter_DEFAULT_CAPTURE_MODE;
            #else
                RefCounter_CONTROL = ctrl;
            #endif /* 0 != RefCounter_CAPTURE_MODE */ 
            
            #endif /* (!RefCounter_ControlRegRemoved) */
        #endif /* (RefCounter_UsingFixedFunction) */
        
        /* Clear all data in the FIFO's */
        #if (!RefCounter_UsingFixedFunction)
            RefCounter_ClearFIFO();
        #endif /* (!RefCounter_UsingFixedFunction) */
        
        /* Set Initial values from Configuration */
        RefCounter_WritePeriod(RefCounter_INIT_PERIOD_VALUE);
        #if (!(RefCounter_UsingFixedFunction && (CY_PSOC5A)))
            RefCounter_WriteCounter(RefCounter_INIT_COUNTER_VALUE);
        #endif /* (!(RefCounter_UsingFixedFunction && (CY_PSOC5A))) */
        RefCounter_SetInterruptMode(RefCounter_INIT_INTERRUPTS_MASK);
        
        #if (!RefCounter_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            (void)RefCounter_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            RefCounter_WriteCompare(RefCounter_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            RefCounter_interruptState = CyEnterCriticalSection();
            
            RefCounter_STATUS_AUX_CTRL |= RefCounter_STATUS_ACTL_INT_EN_MASK;
            
            /* Exit Critical Region*/
            CyExitCriticalSection(RefCounter_interruptState);
            
        #endif /* (!RefCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: RefCounter_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
*******************************************************************************/
void RefCounter_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (RefCounter_UsingFixedFunction)
        RefCounter_GLOBAL_ENABLE |= RefCounter_BLOCK_EN_MASK;
        RefCounter_GLOBAL_STBY_ENABLE |= RefCounter_BLOCK_STBY_EN_MASK;
    #endif /* (RefCounter_UsingFixedFunction) */  
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!RefCounter_ControlRegRemoved || RefCounter_UsingFixedFunction)
        RefCounter_CONTROL |= RefCounter_CTRL_ENABLE;                
    #endif /* (!RefCounter_ControlRegRemoved || RefCounter_UsingFixedFunction) */
    
}


/*******************************************************************************
* Function Name: RefCounter_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  RefCounter_initVar: Is modified when this function is called for the  
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void RefCounter_Start(void) 
{
    if(RefCounter_initVar == 0u)
    {
        RefCounter_Init();
        
        RefCounter_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    RefCounter_Enable();        
}


/*******************************************************************************
* Function Name: RefCounter_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
*******************************************************************************/
void RefCounter_Stop(void) 
{
    /* Disable Counter */
    #if(!RefCounter_ControlRegRemoved || RefCounter_UsingFixedFunction)
        RefCounter_CONTROL &= ((uint8)(~RefCounter_CTRL_ENABLE));        
    #endif /* (!RefCounter_ControlRegRemoved || RefCounter_UsingFixedFunction) */
    
    /* Globally disable the Fixed Function Block chosen */
    #if (RefCounter_UsingFixedFunction)
        RefCounter_GLOBAL_ENABLE &= ((uint8)(~RefCounter_BLOCK_EN_MASK));
        RefCounter_GLOBAL_STBY_ENABLE &= ((uint8)(~RefCounter_BLOCK_STBY_EN_MASK));
    #endif /* (RefCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: RefCounter_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
*******************************************************************************/
void RefCounter_SetInterruptMode(uint8 interruptsMask) 
{
    RefCounter_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: RefCounter_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
*******************************************************************************/
uint8   RefCounter_ReadStatusRegister(void) 
{
    return RefCounter_STATUS;
}


#if(!RefCounter_ControlRegRemoved)
/*******************************************************************************
* Function Name: RefCounter_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
uint8   RefCounter_ReadControlRegister(void) 
{
    return RefCounter_CONTROL;
}


/*******************************************************************************
* Function Name: RefCounter_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
void    RefCounter_WriteControlRegister(uint8 control) 
{
    RefCounter_CONTROL = control;
}

#endif  /* (!RefCounter_ControlRegRemoved) */


#if (!(RefCounter_UsingFixedFunction && (CY_PSOC5A)))
/*******************************************************************************
* Function Name: RefCounter_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
*******************************************************************************/
void RefCounter_WriteCounter(uint32 counter) \
                                   
{
    #if(RefCounter_UsingFixedFunction)
        /* assert if block is already enabled */
        CYASSERT (0u == (RefCounter_GLOBAL_ENABLE & RefCounter_BLOCK_EN_MASK));
        /* If block is disabled, enable it and then write the counter */
        RefCounter_GLOBAL_ENABLE |= RefCounter_BLOCK_EN_MASK;
        CY_SET_REG16(RefCounter_COUNTER_LSB_PTR, (uint16)counter);
        RefCounter_GLOBAL_ENABLE &= ((uint8)(~RefCounter_BLOCK_EN_MASK));
    #else
        CY_SET_REG24(RefCounter_COUNTER_LSB_PTR, counter);
    #endif /* (RefCounter_UsingFixedFunction) */
}
#endif /* (!(RefCounter_UsingFixedFunction && (CY_PSOC5A))) */


/*******************************************************************************
* Function Name: RefCounter_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) The present value of the counter.
*
*******************************************************************************/
uint32 RefCounter_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    #if(RefCounter_UsingFixedFunction)
		(void)CY_GET_REG16(RefCounter_COUNTER_LSB_PTR);
	#else
		(void)CY_GET_REG8(RefCounter_COUNTER_LSB_PTR_8BIT);
	#endif/* (RefCounter_UsingFixedFunction) */
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    #if(RefCounter_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(RefCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG24(RefCounter_STATICCOUNT_LSB_PTR));
    #endif /* (RefCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: RefCounter_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint32) Present Capture value.
*
*******************************************************************************/
uint32 RefCounter_ReadCapture(void) 
{
    #if(RefCounter_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(RefCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG24(RefCounter_STATICCOUNT_LSB_PTR));
    #endif /* (RefCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: RefCounter_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint32) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
*******************************************************************************/
void RefCounter_WritePeriod(uint32 period) 
{
    #if(RefCounter_UsingFixedFunction)
        CY_SET_REG16(RefCounter_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG24(RefCounter_PERIOD_LSB_PTR, period);
    #endif /* (RefCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: RefCounter_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) Present period value.
*
*******************************************************************************/
uint32 RefCounter_ReadPeriod(void) 
{
    #if(RefCounter_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(RefCounter_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG24(RefCounter_PERIOD_LSB_PTR));
    #endif /* (RefCounter_UsingFixedFunction) */
}


#if (!RefCounter_UsingFixedFunction)
/*******************************************************************************
* Function Name: RefCounter_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
*******************************************************************************/
void RefCounter_WriteCompare(uint32 compare) \
                                   
{
    #if(RefCounter_UsingFixedFunction)
        CY_SET_REG16(RefCounter_COMPARE_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG24(RefCounter_COMPARE_LSB_PTR, compare);
    #endif /* (RefCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: RefCounter_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint32) Present compare value.
*
*******************************************************************************/
uint32 RefCounter_ReadCompare(void) 
{
    return (CY_GET_REG24(RefCounter_COMPARE_LSB_PTR));
}


#if (RefCounter_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: RefCounter_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void RefCounter_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    RefCounter_CONTROL &= ((uint8)(~RefCounter_CTRL_CMPMODE_MASK));
    
    /* Write the new setting */
    RefCounter_CONTROL |= compareMode;
}
#endif  /* (RefCounter_COMPARE_MODE_SOFTWARE) */


#if (RefCounter_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: RefCounter_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void RefCounter_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    RefCounter_CONTROL &= ((uint8)(~RefCounter_CTRL_CAPMODE_MASK));
    
    /* Write the new setting */
    RefCounter_CONTROL |= ((uint8)((uint8)captureMode << RefCounter_CTRL_CAPMODE0_SHIFT));
}
#endif  /* (RefCounter_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: RefCounter_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
*******************************************************************************/
void RefCounter_ClearFIFO(void) 
{

    while(0u != (RefCounter_ReadStatusRegister() & RefCounter_STATUS_FIFONEMP))
    {
        (void)RefCounter_ReadCapture();
    }

}
#endif  /* (!RefCounter_UsingFixedFunction) */


/* [] END OF FILE */

