/* ========================================
 *
 * Copyright 2018 Hendrik Lipka
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ========================================
*/

/*
USAGE
- use a CY8CKIT-050 kit
- when using another kit, check the pin connections
- you need an external reference oscillator (an OCXO or TCXO for better stability)
  - its possible to change the project to use a clock derived from the MASTER_CLK (use 10MHz)
- change the referebnce clock frequency below
- on the -050 kit:
  - set the voltage to 5V (you can use 3.3V, but need to change the system settings in the project)
  - connect the external oscillator to P0[0] - don't forget to connect ground too
  - connect P0[1] and P0[2] together to use the provided test signal (derived from the 32kHz crystal)
  - connect LED1 to P0[3] (used for status debugging)
- when measuring external signals, they must be TTL or CMOS, and nit higher than the power supply voltage
- the external signal must be connected to P0[1]
*/
#include <device.h>

#include <stdio.h>

#define STATUS_STOP 1
#define STATUS_INPUT_OVERFLOW 1000
#define STATUS_REF_OVERFLOW 1001

// Change this to your external oscillator frequency
#define REF_FREQ 13000000.0

#define REF_PERIOD 16777215

volatile int status=0;

CY_ISR_PROTO(input_stop);
CY_ISR_PROTO(input_overflow);
CY_ISR_PROTO(ref_overflow);

CY_ISR(input_stop)
{
	isr_InputStop_ClearPending();
	status=STATUS_STOP;
}

CY_ISR(input_overflow)
{
	isr_InputOverflow_ClearPending();
	status=STATUS_INPUT_OVERFLOW;
}

CY_ISR(ref_overflow)
{
	isr_RefOverflow_ClearPending();
	status=STATUS_REF_OVERFLOW;
}

int count=0;

int main()
{
	LED1_Write(1);

    Control_Reg_Start_Write(0);
	
	isr_InputOverflow_StartEx(&input_overflow);
	isr_RefOverflow_StartEx(&ref_overflow);
	isr_InputStop_StartEx(&input_stop);

    CyGlobalIntEnable;

	InputCounter_Start();
	RefCounter_Start();

    LCD_Start();
    LCD_DisplayOn();
    LCD_ClearDisplay();

    // for test purposes uses ILO with 32kHz crystal
	TestClock_Start();
    LCD_PrintString("starting up");	
	LED1_Write(0);

    for(;;)
    {
		LED1_Write(0);

        status=0;
		Control_Reg_Start_Write(1);
		CyDelay(10);
		Control_Reg_Start_Write(0);
		
		LED1_Write(1);

		while (0==status);

		LED1_Write(0);
		
		LCD_ClearDisplay();
		LCD_Position(0,8);
		LCD_PrintDecUint16(status);
		LCD_Position(1,0);
		if (STATUS_STOP==status)
		{
			char buffer[14];
			double freq;
			uint32 ref_count=RefCounter_ReadCounter();
			uint32 in_count=InputCounter_ReadCounter();
			// actual counter value of ref counter is period + current value
			ref_count=ref_count+REF_PERIOD;
			freq=((double)in_count)*((double)REF_FREQ)/((double)ref_count);
			sprintf(buffer,"f=%lu",(uint32)freq);
			LCD_PrintString(buffer);
		}
		else if (STATUS_INPUT_OVERFLOW==status)
		{
			LCD_PrintString("freq too high");
		}
		else if (STATUS_REF_OVERFLOW==status)
		{
			LCD_PrintString("freq too low");
		}
        else
        {
			LCD_PrintString("N/A");
        }            
		count=(count+1)%10000;
		LCD_Position(0,0);
		LCD_PrintDecUint16(count);

        status=0;
    }
}

/* [] END OF FILE */
